@@ -3,70 +3,70 @@
 """
 Copyright 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import weakref
+from typing import Optional, TYPE_CHECKING, List, Dict, Union, Any, cast
 
-from qt import QGraphicsRectItem
-from tile import Tile
+from qt import QGraphicsRectItem, QColor
+from tile import Tile, TileList, Meld, MeldList
 from tileset import Tileset
 from uitile import UITile
-from meld import Meld, MeldList
 from hand import Hand
 from board import Board
 from sound import Sound
 
 from log import logDebug
-from common import Internal, Debug, isAlive, StrMixin
+from common import Internal, Debug, isAlive, ReprMixin
+
+if TYPE_CHECKING:
+    from qt import QGraphicsSceneDragDropEvent
+    from visible import VisiblePlayer
+    from scene import PlayingScene
+    from player import PlayingPlayer
 
 
-class TileAttr(StrMixin):
+class TileAttr(ReprMixin):
 
     """a helper class for syncing the hand board, holding relevant
     tile attributes.
-    xoffset and yoffset are expressed in number of tiles but may be
+    xoffset is expressed in number of tiles but may be
     fractional for adding distances between melds"""
 
-    def __init__(self, hand, meld=None, idx=None, xoffset=None, yoffset=None):
+    def __init__(self, hand:Union['HandBoard', UITile], meld:Optional[Meld]=None, idx:Optional[int]=None,
+        xoffset:Optional[float]=None, yoffset:Optional[int]=None) ->None:
         if isinstance(hand, UITile):
+# TODO only used for Bonus tiles, should split or eliminate this
             self.tile = hand.tile
             self.xoffset = hand.xoffset
             self.yoffset = hand.yoffset
             self.dark = hand.dark
             self.focusable = hand.focusable
         else:
+            assert meld
+            assert idx is not None
             self.tile = Tile(meld[idx])
-            self.xoffset = xoffset
+            assert xoffset is not None # FIXME: overload __init__
+            assert yoffset is not None
+            self.xoffset = xoffset  # type:ignore[assignment]
             self.yoffset = yoffset
             self.dark = self.setDark()
             # dark and focusable are different in a ScoringHandBoard
             self.focusable = self.setFocusable(hand, meld, idx)
-            if self.tile in Debug.focusable:
-                logDebug('TileAttr %s:%s' % (self.tile, self.focusable))
+            if self.tile.name2() in Debug.focusable:
+                logDebug(f'TileAttr {self.tile}:{self.focusable}')
 
-    def setDark(self):
+    def setDark(self) ->bool:
         """should the tile appear darker?"""
-        if self.yoffset == 0:
-            return self.tile.isConcealed
-        else:
-            return not self.tile.isKnown
+        return self.tile.isConcealed if self.yoffset == 0 else not self.tile.isKnown
 
-    def setFocusable(self, hand, meld, dummyIdx):
+    def setFocusable(self, hand:'HandBoard', meld:Meld, idx:Optional[int]) ->bool: # pylint: disable=unused-argument
         """is it focusable?"""
         player = hand.player
+        assert player
+        assert player.game
         return (
             not self.tile.isBonus
             and self.tile.isKnown
@@ -74,139 +74,140 @@ class TileAttr(StrMixin):
             and player == player.game.myself
             and meld.isConcealed and not meld.isKong)
 
-    def __str__(self):
+    def __str__(self) ->str:
+        assert self.xoffset is not None
         return (
-            '%s %.2f/%.1f%s%s' %
-            (self.tile, self.xoffset, self.yoffset,
-             ' dark' if self.dark else '',
-             ' focusable' if self.focusable else ''))
+            f"{self.tile} {self.xoffset:.2f}/{int(self.yoffset)}{' dark' if self.dark else ''}"
+            f"{' focusable' if self.focusable else ''}")
 
 
 class HandBoard(Board):
 
     """a board showing the tiles a player holds"""
-    # pylint: disable=too-many-public-methods,too-many-instance-attributes
-    tileAttrClass = TileAttr
-    penColor = 'blue'
 
-    def __init__(self, player):
+    tileAttrClass = TileAttr
+    penColor = QColor('blue')
+    showShadowsBetweenRows = True
+
+    def __init__(self, player:'VisiblePlayer') ->None:
         assert player
         self._player = weakref.ref(player)
-        self.exposedMeldDistance = 0.15
-        self.concealedMeldDistance = 0.0
-        self.lowerY = 1.0
+        self.exposedMeldDistance:float = 0.15
+        self.concealedMeldDistance:float = 0.0
         Board.__init__(self, 15.6, 2.0, Tileset.current())
         self.isHandBoard = True
         self.tileDragEnabled = False
         self.setParentItem(player.front)
         self.setPosition()
         self.setAcceptDrops(True)
+        assert Internal.Preferences
         Internal.Preferences.addWatch(
             'rearrangeMelds', self.rearrangeMeldsChanged)
-        self.rearrangeMeldsChanged(None, Internal.Preferences.rearrangeMelds)
+        self.rearrangeMeldsChanged(False, bool(Internal.Preferences.rearrangeMelds))
         Internal.Preferences.addWatch(
             'showShadows', self.showShadowsChanged)
 
-    def computeRect(self):
+    def computeRect(self) ->None:
         """also adjust the scale for maximum usage of space"""
         Board.computeRect(self)
-        sideRect = self.player.front.boundingRect()
-        boardRect = self.boundingRect()
-        scale = ((sideRect.width() + sideRect.height())
-                 / (boardRect.width() - boardRect.height()))
-        self.setScale(scale)
+        if self.player:
+            sideRect = self.player.front.boundingRect()
+            boardRect = self.boundingRect()
+            scale = ((sideRect.width() + sideRect.height())
+                     / (boardRect.width() - boardRect.height()))
+            self.setScale(scale)
 
     @property
-    def player(self):
+    def player(self) ->Optional['VisiblePlayer']:
         """player is readonly and never None"""
-        if self._player:
-            return self._player()
+        return self._player() if self._player else None
 
     # this is ordered such that pylint does not complain about
     # identical code in board.py
 
-    @property
-    def name(self):
+    def debug_name(self) ->str:
         """for debugging messages"""
+        if self.player is None:
+            return 'None'
         return self.player.name
 
-    def showShadowsChanged(self, dummyOldValue, dummyNewValue):
+    def showShadowsChanged(self, unusedOldValue:bool, newValue:bool) ->None:
         """Add or remove the shadows."""
         self.setPosition()
 
-    def setPosition(self):
+    def setPosition(self) ->None:
         """Position myself"""
-        show = Internal.Preferences.showShadows
+        assert Internal.Preferences
+        show = bool(Internal.Preferences.showShadows)
         if show:
-            self.setPos(yHeight=1.5)
+            self.setTilePos(yHeight=1.5)
         else:
-            self.setPos(yHeight=1.0)
-        if show:
-            self.lowerY = 1.2
-        else:
-            self.lowerY = 1.0
-        self.setRect(15.6, 1.0 + self.lowerY)
+            self.setTilePos(yHeight=1.0)
+        self.setBoardRect(15.6, 2.2 if show else 2.0)
         self._reload(self.tileset, showShadows=show)
         self.sync()
 
-    def rearrangeMeldsChanged(self, dummyOldValue, newValue):
+    def rearrangeMeldsChanged(self, unusedOldValue:bool, newValue:bool) ->None:
         """when True, concealed melds are grouped"""
         self.concealedMeldDistance = (
             self.exposedMeldDistance if newValue else 0.0)
         self._reload(self.tileset, self._lightSource)
         self.sync()
 
-    def focusRectWidth(self):
+    def focusRectWidth(self) ->int:
         """how many tiles are in focus rect? We want to focus
         the entire meld"""
         # playing game: always make only single tiles selectable
         return 1
 
-    def __str__(self):
+    def __str__(self) ->str:
+        assert self.player
         return self.player.scoringString()
 
-    def lowerHalfTiles(self):
-        """returns a list with all single tiles of the lower half melds
+    def lowerHalfTiles(self) ->List[UITile]:
+        """return a list with all single tiles of the lower half melds
         without boni"""
-        return list(x for x in self.uiTiles if x.yoffset > 0 and not x.isBonus)
+        return [x for x in self.uiTiles if x.yoffset > 0 and not x.isBonus]
 
-    def newLowerMelds(self):
+    def newLowerMelds(self) ->MeldList:
         """a list of melds for the hand as it should look after sync"""
+        return MeldList()
 
-    def newTilePositions(self):
-        """returns list(TileAttr) for all tiles except bonus tiles.
+    def listNewTilePositions(self) ->List[TileAttr]:
+        """return list(TileAttr) for all tiles except bonus tiles.
         The tiles are not associated to any board."""
-        result = list()
+        result:List[TileAttr] = []
+        assert self.player
         newUpperMelds = list(self.player.exposedMelds)
         newLowerMelds = self.newLowerMelds()
-        for yPos, melds in ((0, newUpperMelds), (self.lowerY, newLowerMelds)):
+        for yPos, melds in ((0, newUpperMelds), (1, newLowerMelds)):
             meldDistance = (self.concealedMeldDistance if yPos
                             else self.exposedMeldDistance)
-            meldX = 0
+            meldX:float = 0.0
             for meld in melds:
                 for idx in range(len(meld)):
-                    result.append(
-                        self.tileAttrClass(self, meld, idx, meldX, yPos))
+                    result.append(self.tileAttrClass(self, meld, idx, meldX, yPos))
                     meldX += 1
                 meldX += meldDistance
         return sorted(result, key=lambda x: x.yoffset * 100 + x.xoffset)
 
-    def placeBoniInRow(self, bonusTiles, tilePositions, bonusY, keepTogether=True):
+    def placeBoniInRow(self, bonusTiles:List[UITile], tilePositions:List[TileAttr],
+        bonusY:int, keepTogether:bool=True) ->List[TileAttr]:
         """Try to place bonusTiles in upper or in lower row.
         tilePositions are the normal tiles, already placed.
         If there is no space, return None
 
         returns list(TileAttr)"""
-        positions = list(x.xoffset for x in tilePositions if x.yoffset == bonusY)
+        positions = [x.xoffset for x in tilePositions if x.yoffset == bonusY]
         rightmostTileX = max(positions) if positions else 0
         placeBoni = bonusTiles[:]
         while 13 - len(placeBoni) < rightmostTileX + 1 + self.exposedMeldDistance:
             if keepTogether:
-                return list()
+                return []
             placeBoni = placeBoni[:-1]
-        result = list()
+        result = []
         xPos = 13 - len(placeBoni)
-        newBonusTiles = list(self.tileAttrClass(x) for x in placeBoni)
+        newBonusTiles = [self.tileAttrClass(x) for x in placeBoni]
         for bonus in sorted(newBonusTiles, key=lambda x: x.tile.key):
             bonus.xoffset, bonus.yoffset = xPos, bonusY
             bonus.dark = False
@@ -214,43 +215,43 @@ class HandBoard(Board):
             xPos += 1
         return result
 
-    def newBonusPositions(self, bonusTiles, newTilePositions):
-        """returns list(TileAttr)
-        calculate places for bonus tiles. Put them all in one row,
+    def newBonusPositions(self, bonusTiles:List[UITile], newTilePositions:List[TileAttr]) ->List[TileAttr]:
+        """return list(TileAttr)
+        calculate places for bonus tiles. Try to put them all in one row,
         right adjusted. If necessary, extend to the right even
         outside of our board"""
         if not bonusTiles:
-            return list()
+            return []
         bonusTiles = sorted(bonusTiles, key=lambda x: hash(x.tile))
         result = (
-            self.placeBoniInRow(bonusTiles, newTilePositions, 0.0)
+            self.placeBoniInRow(bonusTiles, newTilePositions, 0)
             or
-            self.placeBoniInRow(bonusTiles, newTilePositions, self.lowerY))
+            self.placeBoniInRow(bonusTiles, newTilePositions, 1))
         if not result:
             # we cannot place all bonus tiles in the same row!
-            result = self.placeBoniInRow(bonusTiles, newTilePositions, 0.0, keepTogether=False)
+            result = self.placeBoniInRow(bonusTiles, newTilePositions, 0, keepTogether=False)
             result.extend(self.placeBoniInRow(
-                bonusTiles[len(result):], newTilePositions, self.lowerY, keepTogether=False))
+                bonusTiles[len(result):], newTilePositions, 1, keepTogether=False))
 
         assert len(bonusTiles) == len(result)
         return result
 
-    def placeTiles(self, tiles):
+    def placeTiles(self, tiles:List[UITile]) ->List[UITile]:
         """tiles are all tiles for this board.
         returns a list of those uiTiles which are placed on the board"""
-        oldTiles = dict()
-        oldBonusTiles = dict()
+        oldTiles:Dict[Tile, List[UITile]] = {}
+        oldBonusTiles:Dict[Tile, List[UITile]] = {}
         for uiTile in tiles:
-            assert isinstance(uiTile, UITile)
+            assert isinstance(uiTile, UITile), f'uiTile is {type(uiTile)}'
             if uiTile.isBonus:
                 targetDict = oldBonusTiles
             else:
                 targetDict = oldTiles
-            if uiTile.tile not in targetDict.keys():
-                targetDict[uiTile.tile] = list()
+            if uiTile.tile not in targetDict:
+                targetDict[uiTile.tile] = []
             targetDict[uiTile.tile].append(uiTile)
-        result = dict()
-        newPositions = self.newTilePositions()
+        result:Dict[UITile, TileAttr] = {}
+        newPositions = self.listNewTilePositions()
         for newPosition in newPositions:
             assert isinstance(newPosition.tile, Tile)
             matches = oldTiles.get(newPosition.tile) \
@@ -271,25 +272,24 @@ class HandBoard(Board):
                 match = matches[0]
                 result[match] = newPosition
                 oldTiles[match.tile].remove(match)
-                if not len(oldTiles[match.tile]):
+                if not oldTiles[match.tile]:
                     del oldTiles[match.tile]
         for newBonusPosition in self.newBonusPositions(
-                list(x for x in tiles if x.isBonus), newPositions):
+                [x for x in tiles if x.isBonus], newPositions):
             result[oldBonusTiles[newBonusPosition.tile][0]] = newBonusPosition
         self._avoidCrossingMovements(result)
         for uiTile, newPos in result.items():
             uiTile.level = 0  # for tiles coming from the wall
-            uiTile.tile = newPos.tile
+            uiTile.change_name(newPos.tile)
             uiTile.setBoard(self, newPos.xoffset, newPos.yoffset)
             uiTile.dark = newPos.dark
             uiTile.focusable = newPos.focusable
-        return result.keys()
+        return list(result.keys())
 
-    def _avoidCrossingMovements(self, places):
+    def _avoidCrossingMovements(self, places:Dict[UITile, TileAttr]) ->None:
         """not needed for all HandBoards"""
-        pass
 
-    def sync(self, adding=None):
+    def sync(self, adding:Optional[List[UITile]]=None) ->None:
         """place all tiles in HandBoard.
         adding tiles: their board is where they come from. Those tiles
         are already in the Player tile lists.
@@ -301,12 +301,16 @@ class HandBoard(Board):
             allTiles.extend(adding)
         self.placeTiles(allTiles)
 
-    def checkTiles(self):
+    def checkTiles(self) ->None:
         """does the logical state match the displayed tiles?"""
-        logExposed = list()
-        physExposed = list()
-        logConcealed = list()
-        physConcealed = list()
+        return
+# FIXME: when exactly should I call this? afterQueuedAnimations does not help
+        # test case: scoring game. move meld from exposed to concealed using the mouse
+        logExposed:TileList = TileList()  # pylint: disable=unreachable
+        physExposed:TileList = TileList()
+        logConcealed:TileList = TileList()
+        physConcealed:TileList = TileList()
+        assert self.player
         for tile in self.player.bonusTiles:
             logExposed.append(tile)
         for uiTile in self.uiTiles:
@@ -320,100 +324,101 @@ class HandBoard(Board):
             for meld in self.player.concealedMelds:
                 logConcealed.extend(meld)
         else:
-            logConcealed = sorted(self.player.concealedTiles)
+            logConcealed = TileList(self.player.concealedTiles)
+            # FIXME: oder player.concealedTiles schon immer sortieren, oder desse Type auf TileList aendern
         logExposed.sort()
         physExposed.sort()
         logConcealed.sort()
         physConcealed.sort()
         assert logExposed == physExposed, (
-            '%s: exposed: player %s != hand %s. Check those:%s' %
-            (self.player, logExposed, physExposed,
-             set(logExposed) ^ set(physExposed)))
+            f'{self.player}: exposed: player {logExposed} != hand {physExposed}. '
+            f'Check those:{set(logExposed) ^ set(physExposed)}')
         assert logConcealed == physConcealed, (
-            '%s: concealed: player %s != hand %s' %
-            (self.player, logConcealed, physConcealed))
+            f'{self.player}: concealed: player {logConcealed} != hand {physConcealed}')
 
 
 class PlayingHandBoard(HandBoard):
 
     """a board showing the tiles a player holds"""
-    # pylint: disable=too-many-public-methods,too-many-instance-attributes
 
-    def __init__(self, player):
+    def __init__(self, player:'VisiblePlayer') ->None:
         HandBoard.__init__(self, player)
 
-    def sync(self, adding=None):
+    def sync(self, adding:Optional[List[UITile]]=None) ->None:
         """place all tiles in HandBoard"""
         allTiles = self.uiTiles[:]
         if adding:
             allTiles.extend(adding)
         newTiles = self.placeTiles(allTiles)
         source = adding if adding else newTiles
-        focusCandidates = list(x for x in source
-                               if x.focusable and x.tile.isConcealed)
+        focusCandidates = [x for x in source if x.focusable and x.tile.isConcealed]
         if not focusCandidates:
             # happens if we just exposed a claimed meld
-            focusCandidates = list(x for x in newTiles
-                                   if x.focusable and x.tile.isConcealed)
+            focusCandidates = [x for x in newTiles if x.focusable and x.tile.isConcealed]
         focusCandidates = sorted(focusCandidates, key=lambda x: x.xoffset)
         if focusCandidates:
             self.focusTile = focusCandidates[0]
+        assert Internal.scene
         Internal.scene.handSelectorChanged(self)
-        self.hasFocus = bool(adding)
+        self.hasLogicalFocus = bool(adding)
 
-    @Board.focusTile.setter
-    def focusTile(self, uiTile):  # pylint: disable=arguments-differ
+    @Board.focusTile.setter  # type: ignore
+    def focusTile(self, uiTile):
         Board.focusTile.fset(self, uiTile)
+        assert self.player
+        assert Internal.scene
         if self.player and Internal.scene.clientDialog:
             Internal.scene.clientDialog.focusTileChanged()
 
-    def setEnabled(self, enabled):
+    def setEnabled(self, enabled:bool) ->None:
         """enable/disable this board"""
         if isAlive(self):
             # aborting a running game: the underlying C++ object might
             # already have been destroyed
+            assert self.player
+            assert self.player.game
             self.tileDragEnabled = (
                 enabled
                 and self.player == self.player.game.myself)
             QGraphicsRectItem.setEnabled(self, enabled)
 
-    def dragMoveEvent(self, event):  # pylint: disable=no-self-use
+    def dragMoveEvent(self, event:'QGraphicsSceneDragDropEvent') ->None:
         """only dragging to discard board should be possible"""
         event.setAccepted(False)
 
-    def _avoidCrossingMovements(self, places):
+    def _avoidCrossingMovements(self, places:Dict[UITile, TileAttr]) ->None:
         """"the above is a good approximation but if the board already had more
         than one identical tile they often switch places - this should not
         happen. So for each element, we make sure that the left-right order is
         still the same as before. For this check, ignore all new tiles"""
         movingPlaces = self.__movingPlaces(places)
-        for yOld in 0, self.lowerY:
-            for yNew in 0, self.lowerY:
+        for yOld in 0, 1:
+            for yNew in 0, 1:
                 items = [x for x in movingPlaces.items()
                          if (x[0].board == self)
                          and x[0].yoffset == yOld
                          and x[1] and x[1].yoffset == yNew
                          and not x[0].isBonus]
-                for element in set(x[1].tile for x in items):
+                for element in {x[1].tile for x in items}:
                     items = [x for x in movingPlaces.items()
                              if x[1].tile is element]
                     if len(items) > 1:
-                        oldList = sorted(list(x[0] for x in items),
+                        oldList = sorted((x[0] for x in items),
                                          key=lambda x:
                                          bool(x.board != self) * 1000 + x.xoffset)
-                        newList = sorted(list(x[1] for x in items),
+                        newList = sorted((x[1] for x in items),
                                          key=lambda x: x.xoffset)
                         for idx, oldTile in enumerate(oldList):
                             places[oldTile] = newList[idx]
 
-    def __movingPlaces(self, places):
+    def __movingPlaces(self, places:Dict[UITile, TileAttr]) ->Dict[UITile, TileAttr]:
         """filter out the left parts of the rows which do not change
         at all"""
-        rows = [[], []]
-        for idx, yOld in enumerate([0, self.lowerY]):
+        rows:List[List[Any]] = [[], []]
+        for idx, yOld in enumerate([0, 1]):
             rowPlaces = [x for x in places.items() if x[0].yoffset == yOld]
             rowPlaces = sorted(rowPlaces, key=lambda x: x[0].xoffset)
-            smallestX = 999
+            smallestX = 999.9
             for tileItem, newPos in places.items():
                 if (tileItem.xoffset != newPos.xoffset
                         or tileItem.yoffset != newPos.yoffset):
@@ -428,23 +433,24 @@ class PlayingHandBoard(HandBoard):
         result.update(dict(rows[1]))
         return result
 
-    def newLowerMelds(self):
+    def newLowerMelds(self) ->MeldList:
         """a list of melds for the hand as it should look after sync"""
+        assert self.player
         if self.player.concealedMelds:
             result = MeldList(self.player.concealedMelds)
         elif self.player.concealedTiles:
-            tileStr = 'R' + ''.join(x for x in self.player.concealedTiles)
-            content = Hand(self.player, tileStr)
+            content = Hand(self.player, unusedTiles=self.player.concealedTiles)
             result = MeldList(content.melds + content.bonusMelds)
         else:
-            return []
+            return MeldList()
+        assert Internal.Preferences
         if not Internal.Preferences.rearrangeMelds:
             result = MeldList(Meld(x) for x in result.tiles())
             # one meld per tile
         result.sort()
         return result
 
-    def discard(self, tile):
+    def discard(self, tile:Tile) ->None:
         """select the rightmost matching tileItem and move it
         to DiscardBoard"""
         if self.focusTile and self.focusTile.tile is tile:
@@ -456,12 +462,15 @@ class PlayingHandBoard(HandBoard):
             # right end of the hand# thus minimizing tile movement
             # within the hand
             lastDiscard = matchingTiles[-1]
-        Internal.scene.discardBoard.discardTile(lastDiscard)
+        assert Internal.scene
+        cast('PlayingScene', Internal.scene).discardBoard.discardTile(lastDiscard)
         for uiTile in self.uiTiles:
             uiTile.focusable = False
 
-    def addUITile(self, uiTile):
+    def addUITile(self, uiTile:UITile) ->None:
         """add uiTile to this board"""
         Board.addUITile(self, uiTile)
+        assert self.player
+        assert self.player.game
         if uiTile.isBonus and not self.player.game.isScoringGame():
             Sound.bonus()
